
__version__ = '0.1.1'


def alias(impl, alias):
    'create a new property "alias" with identical implementation'


class Teritas(object):
    '''An executable implementation of _ in function definitions.

    Attempts to implement all generic interfaces and, when possible,
    always evaluate to True. Inspired by mock._ANY and
    fjord.base.tests.EqualAnything
    '''

    def _void(self, *args, **kwargs):
        return

    def _self(self, *args, **kwargs):
        return self

    def _true(self, *args, **kwargs):
        return True

    def _iterator(self, *args, **kwargs):
        return iter((Teritas(),))

    # basic customization

    __init__ = _void
    __next__ = _self

    def __repr__(self):
        return '<class Teritas>'

    def __bytes__(self):
        return repr(self).encode('ascii')

    # ordering
    # mostly handled by @total_ordering
    __eq__ = _true
    __ne__ = _true
    __lt__ = _true
    __le__ = _true
    __gt__ = _true
    __ge__ = _true

    __hash__ = _true
    __bool__ = _true

    # attribute access
    __getattr__ = _self

    __setattr__ = _true
    __delattr__ = _true

    # callable
    __call__ = _self

    # container types
    def __len__(self):
        return 1

    __getitem__ = _self
    # no need for __missing__
    __setitem__ = _true
    __delitem__ = _true

    __iter__ = _iterator

    __reversed__ = _self
    __contains__ = _self

    # numeric types

    def _firstarg(self, *args, **kwargs):
        return args[0]

    # binary ops
    __add__ = _firstarg
    __sub__ = _firstarg
    __mul__ = _firstarg
    __matmul__ = _firstarg
    __truediv__ = _firstarg
    __floordiv__ = _firstarg
    __mod__ = _firstarg
    __divmod__ = _firstarg
    __pow__ = _firstarg
    __lshift__ = _firstarg
    __rshift__ = _firstarg
    __and__ = _firstarg
    __xor__ = _firstarg
    __or__ = _firstarg

    # swapped operands
    __radd__ = _firstarg
    __rsub__ = _firstarg
    __rmul__ = _firstarg
    __rmatmul__ = _firstarg
    __rtruediv__ = _firstarg
    __rfloordiv__ = _firstarg
    __rmod__ = _firstarg
    __rdivmod__ = _firstarg
    __rpow__ = _firstarg
    __rlshift__ = _firstarg
    __rrshift__ = _firstarg
    __rand__ = _firstarg
    __rxor__ = _firstarg
    __ror__ = _firstarg

    # in-place operations
    __iadd__ = _self
    __isub__ = _self
    __imul__ = _self
    __imatmul__ = _self
    __itruediv__ = _self
    __ifloordiv__ = _self
    __imod__ = _self
    # there is no __idivmod__
    __ipow__ = _self
    __ilshift__ = _self
    __irshift__ = _self
    __iand__ = _self
    __ixor__ = _self
    __ior__ = _self

    # unary arithmetic operations
    __neg__ = _self
    __pos__ = _self
    __abs__ = _self
    __invert__ = _self

    __complex__ = _self
    __int__ = _self
    __float__ = _self
    __round__ = _self

    # operator.index()
    __index__ = _self

    # with statement context managers
    __enter__ = _self
    __exit__ = _void

    # coroutine objects
    __await__ = _iterator


_ = Teritas()



#!/usr/bin/env python

import os
import re
import sys
from setuptools import setup, find_packages


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


readme = open('README.rst').read()

def get_version():
    VERSIONFILE = os.path.join('veritas', '__init__.py')
    VSRE = r"""^__version__ = ['"]([^'"]*)['"]"""
    version_file = open(VERSIONFILE, 'rt').read()
    return re.search(VSRE, version_file, re.M).group(1)


setup(
    name="veritas",
    version=get_version(),
    description="an executable implementation of _ in function definitions",
    long_description=readme,
    license="BSD",
    author="lonnen",
    url='https://github.com/lonnen/veritas',
    packages=find_packages(),
    install_requires=[],
    keywords='',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from decimal import Decimal as D
from fractions import Fraction as F

import pytest

from veritas import Teritas

class TestTeritas:

    def test_basic_customization(self):
        # init
        assert Teritas
        assert Teritas()
        assert Teritas('any', 'arguments', (2,3,4), dir(type))

        _ = Teritas()

        # repr
        assert repr(_) == '<class Teritas>'

        # str
        assert str(_) == '<class Teritas>'

        # TODO: bytes
        assert bytes(_) == b'<class Teritas>'

        # format
        assert format(_) == '<class Teritas>'


    def test_ordering(self):
        _ = Teritas()

        test_values = [
            float('-inf'),
            D('-1e425000000'),
            -1e308,
            F(-22, 7),
            -3.14,
            -2,
            0.0,
            1e-320,
            True,
            F('1.2'),
            D('1.3'),
            float('1.4'),
            F(275807, 195025),
            D('1.414213562373095048801688724'),
            F(114243, 80782),
            F(473596569, 84615),
            7e200,
            D('infinity'),
            Teritas,
            Teritas()
        ]
        for test_value in test_values:
            assert _ < test_value
            assert _ <= test_value
            assert _ == test_value
            assert _ != test_value
            assert _ > test_value
            assert _ >= test_value

        assert hash(_)
        assert hash(_) == True

        assert bool(_) == True


    def test_attribute_access(self):
        _ = Teritas()
        assert _.any_method() == True

        _.any_other_method = lambda x: False
        assert _.any_other_method() == True

        del _.yet_another_method
        assert _.yet_another_method() == True

        # Teritas is returned, which means ordering rules work!
        assert _.final_method() != True
        assert _.final_method() == True
        assert _.final_method() < True
        assert _.final_method() > True

        assert repr(_.final_method()) == '<class Teritas>'


    def test_callable(self):
        assert Teritas
        assert Teritas()
        assert Teritas()()
        assert Teritas('any', 'arguments')('=')(true=True)


    def test_container(self):
        _ = Teritas()

        assert len(_)

        assert _[1]
        assert _[3:4]
        _[0] = False
        assert _[0]

        del _[3]
        assert _[3]

        for x in _:
            assert x



    def test_numeric_types(self):
        '''binary ops, bitwise ops,
        '''
        _ = Teritas()

        test_values = [
            float('-inf'),
            #D('-1e425000000'),
            -1e308,
            -3.14,
            -2,
            1e-320,
            7e200
        ]
        for test_value in test_values:
            assert _ + test_value
            assert _ - test_value
            assert _ * test_value
            assert _ / test_value
            assert _ // test_value
            assert _ % test_value
            assert divmod(_, test_value)
            assert pow(_, test_value)
            assert _ ** test_value

            # associative?
            assert test_value + _
            assert test_value - _
            assert test_value * _
            assert test_value / _
            assert test_value // _
            assert test_value % _
            assert divmod(test_value, _)
            assert pow(test_value, _)
            assert test_value ** _

            # rsub
            _ += test_value
            _ -= test_value
            _ *= test_value
            _ /= test_value
            _ //= test_value
            _ %= test_value
            _ **= test_value

        assert _ << 1
        assert _ >> 1
        assert _ & 1
        assert _ ^ 1
        assert _ | 1

        assert 1 & _
        assert 1 ^ _
        assert 1 | _

        _ &= 1
        _ ^= 1
        _ |= 1

        assert abs(_)
        assert -_
        assert +_
        assert ~_


    def test_context_manager(self):
        with Teritas() as x:
            assert x
